from weishijiajie.base64_custom import Base64
import urllib.parse
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5
from Crypto.Signature import pkcs1_15
from Crypto.Hash import MD5
from Crypto import Random
import collections
from typing import Dict
from base64_custom import Base64Utils

class RSAUtil:
    @staticmethod
    def sign(public_map: Dict[str, object], private_key: str) -> str:
        keys = sorted(k for k in public_map if k.lower() != "sign")
        values = [f"{k}={public_map[k]}" for k in keys]
        public_req_str = "&".join(values)

        key_bytes = Base64Utils.decode(private_key)
        private_key_obj = RSA.import_key(key_bytes)
        signer = pkcs1_15.new(private_key_obj)
        digest = MD5.new(public_req_str.encode())
        signature = signer.sign(digest)
        return Base64Utils.encode(signature)

    @staticmethod
    def verify(public_map: Dict[str, object], public_key: str) -> bool:
        keys = sorted(k for k in public_map if k.lower() != "sign")
        values = [f"{k}={public_map[k]}" for k in keys]
        public_req_str = "&".join(values)
        sign = urllib.parse.unquote(public_map["sign"])

        key_bytes = Base64Utils.decode(public_key)
        public_key_obj = RSA.import_key(key_bytes)
        verifier = pkcs1_15.new(public_key_obj)
        digest = MD5.new(public_req_str.encode())

        try:
            verifier.verify(digest, Base64Utils.decode(sign))
            return True
        except (ValueError, TypeError):
            return False

    @staticmethod
    def decrypt_by_public_key(encrypted: str, public_key: str) -> str:
        encrypted_data = Base64Utils.decode(encrypted)
        key_bytes = Base64Utils.decode(public_key)
        public_key_obj = RSA.import_key(key_bytes)
        cipher = PKCS1_v1_5.new(public_key_obj)

        sentinel = Random.new().read(15)
        decrypted_data = cipher.decrypt(encrypted_data, sentinel)
        return decrypted_data.decode()

    @staticmethod
    def encrypt_by_private_key(source: str, private_key: str) -> str:
        data = source.encode()
        key_bytes = Base64Utils.decode(private_key)
        private_key_obj = RSA.import_key(key_bytes)
        cipher = PKCS1_v1_5.new(private_key_obj)
        encrypted_data = cipher.encrypt(data)
        return Base64Utils.encode(encrypted_data)

